#include "Fonts.h"
#include <QtCore/QObject>
#include <QtGui/QFontInfo>
#include <stdio.h>

#define STYLE_IMAGE(s)                       \
   ((s) == QFont::StyleNormal  ? "Normal"  : \
    (s) == QFont::StyleItalic  ? "Italic"  : \
    (s) == QFont::StyleOblique ? "Oblique" : \
                                 "?")

#define STYLEHINT_IMAGE(s)                               \
   ((s) == QFont::AnyStyle   ? "Any"                   : \
    (s) == QFont::Helvetica  ? "Helvetica/SansSerif"   : \
    (s) == QFont::Times      ? "Times/Serif"           : \
    (s) == QFont::Courier    ? "Courier/TypeWriter"    : \
    (s) == QFont::Decorative ? "Decorative/OldEnglish" : \
    (s) == QFont::System     ? "System"                : \
                               "?")

#ifdef DEBUG_FONTS
static QString  describeFont(const QFont&  f)
{
   QFontInfo  fi(f);
   return QString("      req: %1\n"
                  "           family=%2 pixelSize=%3 pointSize=%4 weight=%5\n"
                  "           %6 %7 %8\n"
                  "           style=%9 styleHint=%10\n"
                  "      got: family=%11 pixelSize=%12 pointSize=%13 weight=%14\n"
                  "           %15 %16 %17\n"
                  "           style=%18 styleHint=%19")
             .arg(f.toString().toStdString().c_str())
             .arg(f.family())
             .arg(f.pixelSize())
             .arg(f.pointSizeF())
             .arg(f.weight())
             .arg(f.bold()       ? "bold"        : "non-bold")
             .arg(f.fixedPitch() ? "fixed-pitch" : "variable-pitch")
             .arg(f.italic()     ? "italic"      : "non-italic")
             .arg(STYLE_IMAGE(f.style()))
             .arg(STYLEHINT_IMAGE(f.styleHint()))
             .arg(fi.family())
             .arg(fi.pixelSize())
             .arg(fi.pointSizeF())
             .arg(fi.weight())
             .arg(fi.bold()       ? "bold"        : "non-bold")
             .arg(fi.fixedPitch() ? "fixed-pitch" : "variable-pitch")
             .arg(fi.italic()     ? "italic"      : "non-italic")
             .arg(STYLE_IMAGE(fi.style()))
             .arg(STYLEHINT_IMAGE(fi.styleHint()));
}
#endif

extern QFont Fonts::makeHeader (const QFont&  p)
{
   QFont result = p;
   result.setFixedPitch(false);
   result.setStyleHint(QFont::Helvetica);
   result.setBold(true);
   if (result.pixelSize() != -1) {
      result.setPixelSize(result.pixelSize() * 3 / 2);
   } else if (result.pointSize() != -1) {
      result.setPointSize(result.pointSize() * 3 / 2);
   }
   return result;
}

extern QFont Fonts::makeFixed (const QFont&  p)
{
#ifdef DEBUG_FONTS
   printf("   makeFixed:\n%s\n",
          describeFont(p).toStdString().c_str());
#endif

   QFont result = p;
   result.setFixedPitch(true);

   static const char*  fixedFamilies[] = { "Bitstream Vera Sans Mono",
                                           "DejaVu Sans Mono",
                                           "Monospace",
                                           "Courier" };
#define NUM_FIXEDFAMILIES  (sizeof(fixedFamilies) / sizeof(fixedFamilies[0]))
   unsigned int  f;
   for (f = 0; f < NUM_FIXEDFAMILIES; f++) {
      QString  fam(QObject::tr(fixedFamilies[f]));
      result.setFamily(fam);
      if (QFontInfo(result).family() == fam) break;
   }

#ifdef DEBUG_FONTS
   printf("         ==>\n%s\n",
          describeFont(result).toStdString().c_str());
#endif
   return result;
}

extern QFont Fonts::makeBold (const QFont&  p)
{
#ifdef DEBUG_FONTS
   printf("   makeBold:\n%s\n",
          describeFont(p).toStdString().c_str());
#endif
   QFont result = p;
   result.setBold(true);
#ifdef DEBUG_FONTS
   printf("         ==>\n%s\n",
          describeFont(result).toStdString().c_str());
#endif
   return result;
}
