#ifndef PackageTableSortFilterProxyModel_h
#define PackageTableSortFilterProxyModel_h

// QRegExp takes about 25x as long as regexec, on average.  The unicode
// support and additional features in QRegExp are much less than important
// than speed in nuu, so use regcomp.
#define USE_REGCOMP

#include "YumIO.h"
#include "HAbstractTableModel.h"
#include "HSortFilterProxyModel.h"
#include "Package.h"
#include "PackageTable.h"
#ifdef USE_REGCOMP
#include <regex.h>
#endif

// QSortFilterProxyModel modified with lessThan function made to understand
// PackageTable columns' values.

namespace PackageTable {

   class SortFilterProxyModel : public HSortFilterProxyModel {
      Q_OBJECT
   public:
      explicit SortFilterProxyModel(StateFilter  stateFilter,
                                    bool         dupsFilter,
                                    bool         groupsFilter,
                                    YumIO*       yumIO,
                                    QObject*     parent       = NULL);
   public:
      void setSourceModel   (HAbstractTableModel*  sourceModel);
   public:
      void setMode          (Mode                mode);
      void setFilter        (bool                filter);
      bool getFilter        ();
      void setFilterRegExp  (const QString&      regExp);
      void setStateFilter   (StateFilter         stateFilter);
      void setDupsFilter    (bool                dupsFilter);
      void setGroupsFilter  (bool                groupsFilter);
   protected:
      bool filterAcceptsRow (int                 source_row, 
                             const QModelIndex&  source_parent) const;
      bool acceptRow        (Package::Entry*     pkg,
                             int                 source_row,
                             const QModelIndex&  source_parent) const;
      bool lessThan         (const QModelIndex&  left, 
                             const QModelIndex&  right)         const;
      bool matches          (const QString&      s)             const;
   public slots:
      void clear();
      void clearCache();
   protected:
      Mode                                 mode;
      bool                                 filter;
      StateFilter                          stateFilter;
      bool                                 dupsFilter;
      bool                                 groupsFilter;
      YumIO*                               yumIO;
      QAbstractItemModel*                  sModel;
      mutable QHash<Package::Entry*,bool>  in;
#ifdef USE_REGCOMP
      int                                  regcomp_status;
      regex_t                              preg;
#endif
   };

};

#endif
