#include "ProgressMinion.h"
#include <QtGui/QGridLayout>
#include <QtGui/QLabel>
#include <QtGui/QToolTip>
#include <QtGui/QColor>
#include <QtGui/qrgb.h>

#define deducedFg(bg) (qGray((bg).rgb()) >= 124 ? Qt::black : Qt::white)

Progress::ThreadsState::ThreadsState()
   : num(0), them(NULL)
{
}

#include <stdio.h>

Progress::ThreadsState::ThreadsState(Threads  num)
   : num(num), them(new ThreadState[num])
{
   Threads  t;
   for (t = 0; t < num; t++) {
      them[t].value = 0;
      them[t].tail  = 1;
      them[t].state = NotStarted;
   }
}

Progress::ThreadsState::ThreadsState(const ThreadsState&  orig)
{
   num = orig.num;
   if (orig.them != NULL) {
      them = new ThreadState[num];
      Threads  t;
      for (t = 0; t < num; t++) {
         them[t].activity = orig.them[t].activity;
         them[t].value    = orig.them[t].value;
         them[t].tail     = orig.them[t].tail;
         them[t].state    = orig.them[t].state;
      }
   } else {
      them = NULL;
   }
}

Progress::ThreadsState&  Progress::ThreadsState::operator=
   (const ThreadsState&  orig)
{
   num = orig.num;
   if (them != NULL) {
      delete [] them;
   }
   if (orig.them != NULL) {
      them = new ThreadState[num];
      Threads  t;
      for (t = 0; t < num; t++) {
         them[t].activity = orig.them[t].activity;
         them[t].value    = orig.them[t].value;
         them[t].tail     = orig.them[t].tail;
         them[t].state    = orig.them[t].state;
      }
   } else {
      them = NULL;
   }

   return *this;
}

Progress::ThreadsState::~ThreadsState()
{ 
   if (them != NULL) {
      delete [] them;
   }
}

Progress::Minion::Minion(HProgressBar*  totalProgress,
                         HProgressBar*  itemProgress)
   : totalProgress(totalProgress),
     itemProgress(itemProgress)
{
   // Beware.  This object isn't necessarily in the main thread yet.
   // Don't try any GUI activities here.

   connect(itemProgress,  SIGNAL(toolTip(const QPoint&)),
           this,          SLOT(itemProgressToolTip(const QPoint&)));
}

#include <stdio.h>

void Progress::Minion::startTotal(QString  activity,
                                Steps    totalTail)
{
   totalProgress->setActivity(activity);
   totalProgress->setMaximum(totalTail);
   totalProgress->setValue(0);
   totalProgress->setEnabled(true);

   itemProgress->reset();
   itemProgress->setEnabled(false);
}

void Progress::Minion::startItem(QString  activity,
                                 Steps    totalValue,
                                 Steps    itemTail)
{
   totalProgress->setValue(totalValue);

   itemProgress->setActivity(activity);
   itemProgress->setMaximum(itemTail);
   itemProgress->setValue(0);
   itemProgress->setEnabled(true);
}

void Progress::Minion::changeItemActivity (QString  activity)
{
   itemProgress->setActivity(activity);
}

void Progress::Minion::changeItemActs (Steps  totalValue)
{
   totalProgress->setValue(totalValue);
}

void Progress::Minion::setItemValue(Steps  totalValue,
                                  Steps  value)
{
   totalProgress->setValue(totalValue);
   itemProgress->setValue(value);
}

void Progress::Minion::endItemActs(Steps  totalValue)
{
   totalProgress->setValue(totalValue);
}

void Progress::Minion::setTotalValue (Steps  value)
{
   totalProgress->setValue(value);
}

void Progress::Minion::updateThreadsState (const ThreadsState&  threads)
{
   this->threads = threads;
}

void Progress::Minion::reset()
{
   totalProgress->setMinimum(0);
   totalProgress->setMaximum(1);
   itemProgress->setMinimum(0);
   itemProgress->setMaximum(1);
   totalProgress->reset();
   itemProgress->reset();
   totalProgress->setEnabled(false);
   itemProgress->setEnabled(false);
}

static QLabel* makeLabel (QString        str,
                          const QColor&  windowColor)
{
   QLabel*  result = new QLabel(str);

   result->ensurePolished();

   result->setFrameShape(QFrame::StyledPanel);
   result->setFrameShadow(QFrame::Plain);

   QPalette  pal = result->palette();
   pal.setColor(QPalette::Window,     windowColor);
   pal.setColor(QPalette::WindowText, deducedFg(windowColor));
   result->setPalette(pal);
   result->setAutoFillBackground(true);

   return result;
}

static QColor notStartedColor (0xff, 0x8c, 0x00); // dark orange
static QColor retryingColor   (0xff, 0x8c, 0x00); // dark orange
static QColor runningColor    (0xff, 0xff, 0x00); // yellow
static QColor doneColor       (0x90, 0xee, 0x90); // palegreen2


void Progress::Minion::setupTip (GeneralTip*  tip,
                                 void*        data)
{
   Minion*  that = (Minion*)data;

   QGridLayout*  grid = new QGridLayout(tip);
   grid->setMargin(4);
   tip->setLayout(grid);

   Threads  notStartedCount = 0;

   int      row = 0;
   Threads  t;
   for (t = 0; t < that->threads.count(); t++) {
      const ThreadState&  ts(that->threads[t]);
      switch (ts.state) {
      case NotStarted:
         notStartedCount++;
         break;
      case Retrying:
         {
            QLabel*  act = new QLabel(ts.activity);
            grid->addWidget(act, row, 0);
            QLabel*  status = makeLabel("Retrying", retryingColor);
            status->setFrameStyle(QFrame::Box);
            grid->addWidget(status, row, 1);
            row++;
         }
         break;
      case Running:
         {
            QLabel*  act = new QLabel(ts.activity);
            grid->addWidget(act, row, 0);
            QLabel*  status;
            if (ts.tail == 1) {
               status = makeLabel("In Progress", runningColor);
            } else {
               // If yum ever starts providing actual progress values when doing
               // parallel grabs, we might want to change the simplistic n/m
               // display with a nice progress bar.
               status = makeLabel(QString("%1 / %2 (%3%%)")
                                     .arg(ts.value)
                                     .arg(ts.tail)
                                     .arg((double)ts.value / ts.tail),
                                  runningColor);
            }
            grid->addWidget(status, row, 1);
            row++;
         }
         break;
      case Done:
         {
            QLabel*  act = new QLabel(ts.activity);
            grid->addWidget(act, row, 0);
            QLabel*  status = makeLabel("Complete", doneColor);
            status->setFrameStyle(QFrame::Box);
            grid->addWidget(status, row, 1);
            row++;
         }
         break;
      }
   }

   if (notStartedCount > 0) {
      QLabel*  act = new QLabel(QString("%1 downloads").arg(notStartedCount));
      grid->addWidget(act, row, 0);
      QLabel*  status = makeLabel("Not Started Yet", notStartedColor);
      grid->addWidget(status, row, 1);
      row++;
   }
}

void Progress::Minion::itemProgressToolTip (const QPoint&  pos)
{
   if (threads.count() == 0) return;

   GeneralTip::showTip(itemProgress->mapToGlobal(pos),
                       setupTip,
                       this,
                       itemProgress,
                       /* permanent => */ true);
}
